print("[RENDER GRAPHICS] render_graphics.py")

import codecs  # used for writing files - more unicode friendly than standard open() module

import shutil
import sys
import os
currentdir = os.curdir
from multiprocessing import Pool
import multiprocessing
logger = multiprocessing.log_to_stderr()
logger.setLevel(25)
from time import time

import iron_horse
import utils
import global_constants

# get args passed by makefile
makefile_args = utils.get_makefile_args(sys)
# default to no mp, makes debugging easier (mp fails to pickle errors correctly)
num_pool_workers = makefile_args.get('num_pool_workers', 0)
if num_pool_workers == 0:
    use_multiprocessing = False
    # just print, no need for a coloured echo_message
    print('Multiprocessing disabled: (PW=0)')
else:
    use_multiprocessing = True
    # just print, no need for a coloured echo_message
    print('Multiprocessing enabled: (PW=' + str(num_pool_workers) + ')')

graphics_input_path = os.path.join(currentdir, 'src', 'graphics')
graphics_output_path = os.path.join(
    iron_horse.generated_files_path, 'graphics')
if os.path.exists(graphics_output_path):
    shutil.rmtree(graphics_output_path)
os.mkdir(graphics_output_path)

hint_file = codecs.open(os.path.join(
    graphics_output_path, '_graphics_files_here_are_generated.txt'), 'w', 'utf8')
hint_file.write("Don't edit the graphics files here.  They're generated by the build script. \n Edit sources in src/graphics.")
hint_file.close()

def run_pipelines(consist):
    if len(consist.gestalt_graphics.pipelines) == 0:
        # no pipelines to run, just copy graphics file
        shutil.copy(os.path.join(graphics_input_path, consist.roster_id,
                                 consist.id + '.png'), graphics_output_path)
    else:
        # run pipelines, obvs
        for pipeline in consist.gestalt_graphics.pipelines:
            result = pipeline.render(consist, global_constants)

def report_sprites_complete(consists):
    # project management eh :P
    complete = len([consist.sprites_complete for consist in consists if consist.sprites_complete])
    print("Sprites complete for", complete, "consists; incomplete for",
          len(consists) - complete, "consists;", str(int(100 * (complete / len(consists)))) + '%')
    incomplete_by_track_type = {}
    for consist in consists:
        if not consist.sprites_complete:
            incomplete_by_track_type.setdefault(consist.base_track_type, []).append(consist)
    for track_type, incomplete_consists in incomplete_by_track_type.items():
        print("  *", track_type, len(incomplete_consists))

# wrapped in a main() function so this can be called explicitly, because unexpected multiprocessing fork bombs are bad
def main():
    start = time()
    consists = iron_horse.get_consists_in_buy_menu_order()

    if use_multiprocessing == False:
        for consist in consists:
            run_pipelines(consist)
    else:
        # Would this go faster if the pipelines from each consist were placed in MP pool, not just the consist?
        # probably potato / potato tbh
        pool = Pool(processes=num_pool_workers)
        pool.map(run_pipelines, consists)
        pool.close()

    report_sprites_complete(consists)

    for dir_name in ['rear_lights']:
        dir_path =  os.path.join(graphics_input_path, dir_name)
        shutil.copytree(dir_path, os.path.join(graphics_output_path, dir_name))

    print(format((time() - start), '.2f') + 's')


if __name__ == '__main__':
    main()
